/*
 * Copyright (c) 2015 Vasa Consulting LLC
 *
 * This file is part of TYPE61CHK.
 *
 * TYPE61CHK is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TYPE61CHK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TYPE61CHK.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdbool.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sysexits.h>

#include "support.h"

int
dichotomize(char *sender, char **user, char **domain)
{
	char   *p;
	int		len = 0;

	p = strchr(sender, '@');
	if (p == NULL)
		return EX_DATAERR;

	len = strlen(sender);

	if ((*user = strndup(sender, (p - sender))) == NULL)
		return EX_OSERR;

	if ((*domain = strndup(p + 1, len - (p - sender) - 1)) == NULL)
		return EX_OSERR;

	return EX_OK;
}

char *
prep_msg(char *file, char *query)
{
	char   *final	= NULL;
	char   *tmp		= NULL;

	final = read_msg_file(file);

	if (final == NULL)
		return NULL;

	tmp = replace_text(final, "{{query}}", query);

	if (tmp != NULL)
	{
		free(final);
		final = tmp;
	}

	return final;
}

char *
read_msg_file(char *file)
{
	char   *msg = NULL;
	char   *tmp = NULL;
	FILE   *fp;
	size_t	total_nbr = 0, nbr; /* num bytes read */
	size_t	btr = 32; /* bytes to read */

	fp = fopen(file, "r");

	if (fp == NULL)
	{
		fprintf(stdout, "no fp, check for files in messages/ dir");
		return NULL;
	}

	msg = malloc(sizeof(char) * btr);
	memset(msg, '\0', sizeof(char) * btr);

	if (msg == NULL)
		return NULL;

	while (! feof(fp))
	{
		nbr = fread(msg + total_nbr, sizeof(char), btr, fp);
		total_nbr += nbr;

		/* If we asked for X bytes and got X bytes, need to realloc */
		if (nbr == btr)
		{
			tmp = realloc(msg, total_nbr + (sizeof(char) * btr));
			if (tmp == NULL)
			{
				free(msg);
				return NULL;
			}

			if (tmp != msg)
				msg = tmp;

			memset(msg + total_nbr, '\0', sizeof(char) * btr);

			/* handle reaching EOF */
			if (nbr < btr)
			{
				btr = (btr - (btr - nbr));
				memset(msg + total_nbr - btr, '\0', btr);
			}
		}

		if (nbr < btr)
			break;
	}

	fclose(fp);

	return msg;
}

char *
replace_text(char *in, char *search, char *replace)
{
	char		   *out = NULL;
	size_t			c, d, e;
	size_t			in_len, out_len, largest_len;
	const size_t	max_len = 1024;
	char		   *buf = NULL;
	size_t			buf_len = 0;
	size_t			buf_pos = 0;

	if (in == NULL || search == NULL || replace == NULL)
		return NULL;

	in_len = strnlen(in, max_len);
	buf_len = strnlen(search, max_len);
	out_len = (sizeof(char) * strnlen(in, max_len))
				 - strnlen(search, max_len)
				 + strnlen(replace, max_len)
				 ;
	largest_len = (in_len > out_len) ? in_len : out_len;

	buf = malloc(sizeof(char) * buf_len);

	if (buf == NULL)
		goto out;

	memset(buf, '\0', buf_len);
	strncpy(buf, search, buf_len);

	/* find the starting location of search */
	for (c = 0; c < in_len; c++)
	{
		if (in[c] == buf[buf_pos])
		{
			buf_pos++;
			if (buf_pos == buf_len - 1)
				break;
		}
		else
			buf_pos = 0;
	}

	if (buf_pos == buf_len - 1)
		c -= buf_pos - 1;
	else
		/* no search match found */
		goto out;

	out = malloc(largest_len + 1);

	if (out == NULL)
		goto out;

	memset(out, '\0', largest_len);
	strncpy(out, in, largest_len);

	/* cp from in[search+strlen(search)] to in[strlen(in)] into new buffer */
	for (d = out_len - 1, e = in_len - 1;
		 d > (c + (strnlen(replace, max_len) - 1));
		 d--, e--)
		memset(&out[d], in[e], 1);

	/* and finally insert replacement text */
	for (d = c, e = 0; d < (c + strnlen(replace, max_len)); d++)
	{
		if (&replace[e] == '\0')
			break;

		memset(&out[d], replace[e], 1);
		e++;
	}

	out[out_len] = '\0';

out:
	if (buf != NULL)
		free(buf);

	return out;
}
