/*
 * Copyright (c) 2015 Vasa Consulting LLC
 *
 * This file is part of TYPE61CHK.
 *
 * TYPE61CHK is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TYPE61CHK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TYPE61CHK.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * Unit testing harness
 */

#include <stdio.h>
#include <string.h>
#include "CUnit/Basic.h"
#include "tests.h"

/* CUnit test suites & test routines */
struct cts {
	char *suite_name;
	int (*init)(void);
	int (*clean)(void);
	struct ctr *ctrs;
};

struct ctr {
	char *routine_name;
	void (*routine)(void);
};

/* test routines */
static struct ctr message_ctrs[] = {
	{ "dichotomize", testMESSAGEDICHOTOMIZE },
	{ "prep_msg", testMESSAGEPREPMSG },
	{ "read_msg_file", testMESSAGEREADMSGFILE },
	{ "replace_text", testMESSAGEREPLACETEXT },
	{ NULL, NULL }
};

static struct ctr curl_ctrs[] = {
	{ "add_msg_to_curl", testCURLADDMSGTOCURL },
	{ "add_recp_to_curl", testCURLADDRECPTOCURL },
	{ "init_curl", testCURLINIT },
	{ NULL, NULL }
};

static struct ctr sha_ctrs[] = {
	{ "init_sha", testSHAINIT },
	{ "hash_user", testSHAHASHUSER },
	{ NULL, NULL }
};

static struct ctr ldns_ctrs[] = {
	{ "fetch_openpgpkey", testLDNSFETCHOPENPGPKEY },
	{ "query_text_ldns", testLDNSQUERYTEXTLDNS },
	{ "free_ldns", testLDNSFREELDNS },
	{ NULL, NULL }
};

static struct ctr gpgme_ctrs[] = {
	{ "init_gpgme", testGPGMEINITGPGME },
	{ "encrypt_with_gpgme", testGPGMEENCRYPTWITHGPGME },
	{ "free_gpgme", testGPGMEFREEGPGME },
	{ NULL, NULL }
};

/* suites */
static struct cts ctss[] = {
	{ "message", init_suite_message, clean_suite_message, message_ctrs },
	{ "curl", init_suite_curl, clean_suite_curl, curl_ctrs },
	{ "sha", init_suite_sha, clean_suite_sha, sha_ctrs },
	{ "ldns", init_suite_ldns, clean_suite_ldns, ldns_ctrs },
	{ "gpgme", init_suite_gpgme, clean_suite_gpgme, gpgme_ctrs },
};

int main(void)
{
	size_t		i;
	size_t		j;
	CU_pSuite	pSuite = NULL;

	if (CUE_SUCCESS != CU_initialize_registry())
		return CU_get_error();

	for (i = 0; i < sizeof(ctss) / sizeof(struct cts); i++)
	{
		pSuite = CU_add_suite(ctss[i].suite_name,
							  ctss[i].init,
							  ctss[i].clean);

		if (NULL == pSuite)
		{
			CU_cleanup_registry();
			return CU_get_error();
		}

		for (j = 0; ; j++)
		{
			if (ctss[i].ctrs[j].routine_name == NULL)
				break;

			if (NULL == CU_add_test(pSuite,
									ctss[i].ctrs[j].routine_name,
									ctss[i].ctrs[j].routine))
			{
				CU_cleanup_registry();
				return CU_get_error();
			}
		}
	}

	/* Run all tests using the CUnit Basic interface */
	CU_basic_set_mode(CU_BRM_VERBOSE);
	CU_basic_run_tests();
	CU_cleanup_registry();
	return CU_get_error();
}

