/*
 * Copyright (c) 2015 Vasa Consulting LLC
 *
 * This file is part of TYPE61CHK.
 *
 * TYPE61CHK is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TYPE61CHK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TYPE61CHK.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <sysexits.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "CUnit/Basic.h"

#include "tests.h"
#include "support.h"

/* Globals provided by environment, mocked here */
static char   *msg_file = "messages/openpgpkey-record-signed.txt";

int
init_suite_message()
{
	return 0;
}

int
clean_suite_message()
{
	return 0;
}

void
testMESSAGEDICHOTOMIZE()
{
	int		retval = EX_OK;
	char   *sender = "alexander@vasaconsulting.com";
	char   *user = NULL;
	char   *domain = NULL;

	retval = dichotomize(sender, &user, &domain);

	CU_ASSERT(EX_OK == retval);
	CU_ASSERT_EQUAL(9, strlen(user));
	CU_ASSERT_NSTRING_EQUAL(user, "alexander", 9);
	CU_ASSERT_EQUAL(18, strlen(domain));
	CU_ASSERT_NSTRING_EQUAL(domain, "vasaconsulting.com", 18);
}

void
testMESSAGEPREPMSG()
{
	char   *msg = NULL;
	char   *query = "2d41ad159bbc21a20a4e05ceb1b0"
					"037aaf7821bc47ec8443af5fa3a7"
					"._openpgpkey.vasaconsulting.com";

	msg = prep_msg(msg_file, query);

	CU_ASSERT_PTR_NOT_NULL_FATAL(msg);
	CU_ASSERT_EQUAL(351, strlen(msg));
	CU_ASSERT_NSTRING_EQUAL(msg, "Hello,", 6);
	/* -11 to eat the two final \n chars */
	CU_ASSERT_NSTRING_EQUAL(msg + 351 - 11, "TYPE61CHK,", 9);
	CU_ASSERT('\0' == msg[351]);
}

void
testMESSAGEREADMSGFILE()
{
	char *final = NULL;

	final = read_msg_file(msg_file);

	CU_ASSERT_PTR_NOT_NULL_FATAL(final);
	CU_ASSERT_EQUAL(273, strlen(final));
	CU_ASSERT_NSTRING_EQUAL(final, "Hello,", 6);
	CU_ASSERT_NSTRING_EQUAL(final + 273 - 11, "TYPE61CHK", 9);
	CU_ASSERT('\0' == final[273]);
}

void
testMESSAGEREPLACETEXT()
{
	char *tmp   = NULL;
	/* test replacement in the middle with equal num chars */
	char *text1 = "abcdefghijklmnopqrstuvwxyz";
	/* test long string replacing shorter one */
	char *text2 = "1234567890";
	/* test short string replacing a longer one */
	char *text3 = "abcdefghijklmnopqrstuvwxyz1234567890";

	tmp = replace_text(text1, "lmnop", "12345");
	CU_ASSERT_PTR_NOT_NULL_FATAL(tmp);
	CU_ASSERT_EQUAL(26, strlen(tmp));
	CU_ASSERT_NSTRING_EQUAL(tmp, "abcdefghijk12345qrstuvwxyz", 26);
	free(tmp);
	tmp = NULL;

	tmp = replace_text(text2, "7890", "ALEXANDER");
	CU_ASSERT_PTR_NOT_NULL_FATAL(tmp);
	CU_ASSERT_EQUAL(15, strlen(tmp));
	CU_ASSERT_NSTRING_EQUAL(tmp, "123456ALEXANDER", 15);
	free(tmp);
	tmp = NULL;

	tmp = replace_text(text3, "klmnopqrst", "SHORT");
	CU_ASSERT_PTR_NOT_NULL_FATAL(tmp);
	CU_ASSERT_EQUAL(31, strlen(tmp));
	CU_ASSERT_NSTRING_EQUAL(tmp, "abcdefghijSHORTuvwxyz1234567890", 31);
	tmp = NULL;
}

